@@ -18,6 +18,7 @@ from .serializer import deserialize
 from .utils import (
     close_fd,
     current_task,
+    exception_in_chain,
     flush_fh,
     kill,
     read_fd,
@@ -227,8 +228,10 @@ class Worker:
                         queue = self.wakaq.queues_by_key[queue_broker_key]
                         current_task.set((task, payload))
                         retry = payload.get("retry") or 0
+
                         try:
                             self._execute_task(task, payload, queue=queue)
+
                         except SoftTimeout:
                             retry += 1
                             max_retries = task.max_retries
@@ -243,8 +246,28 @@ class Worker:
                                 self.wakaq._enqueue_at_end(
                                     task.name, queue.name, payload["args"], payload["kwargs"], retry=retry
                                 )
-                        except:
+
+                        except Exception as e:
+                            if exception_in_chain(e, SoftTimeout):
+                                retry += 1
+                                max_retries = task.max_retries
+                                if max_retries is None:
+                                    max_retries = (
+                                        queue.max_retries if queue.max_retries is not None else self.wakaq.max_retries
+                                    )
+                                if retry > max_retries:
+                                    log.error(traceback.format_exc())
+                                else:
+                                    log.warning(traceback.format_exc())
+                                    self.wakaq._enqueue_at_end(
+                                        task.name, queue.name, payload["args"], payload["kwargs"], retry=retry
+                                    )
+                            else:
+                                log.error(traceback.format_exc())
+
+                        except:  # catch BaseException, SystemExit, KeyboardInterrupt, and GeneratorExit
                             log.error(traceback.format_exc())
+
                         finally:
                             current_task.set(None)
 
@@ -266,7 +289,14 @@ class Worker:
             if current_task.get():
                 raise
 
-        except:
+        except Exception as e:
+            if exception_in_chain(e, SoftTimeout):
+                if current_task.get():
+                    raise
+            else:
+                log.error(traceback.format_exc())
+
+        except:  # catch BaseException, SystemExit, KeyboardInterrupt, and GeneratorExit
             log.error(traceback.format_exc())
 
         finally:
@@ -364,6 +394,7 @@ class Worker:
                     try:
                         self._execute_task(task, payload)
                         break
+
                     except SoftTimeout:
                         retry += 1
                         max_retries = task.max_retries
@@ -374,9 +405,26 @@ class Worker:
                             break
                         else:
                             log.warning(traceback.format_exc())
-                    except:
+
+                    except Exception as e:
+                        if exception_in_chain(e, SoftTimeout):
+                            retry += 1
+                            max_retries = task.max_retries
+                            if max_retries is None:
+                                max_retries = self.wakaq.max_retries
+                            if retry > max_retries:
+                                log.error(traceback.format_exc())
+                                break
+                            else:
+                                log.warning(traceback.format_exc())
+                        else:
+                            log.error(traceback.format_exc())
+                            break
+
+                    except:  # catch BaseException, SystemExit, KeyboardInterrupt, and GeneratorExit
                         log.error(traceback.format_exc())
                         break
+
             finally:
                 current_task.set(None)
 
